package org.ugasp.client.test;

import java.io.File;
import java.util.Vector;

import org.ugasp.client.http.UGASPClientHttp;
import org.ugasp.client.lobby.ApplicationInstanceInfos;


/**
 * This class allow communication between two GASP clients.
 * Depending to the constructor, the client can be the owner of the game instance, or just an actor.
 * If it is the owner, it will create the game instance and wait for another client to start it. 
 * Otherwise, it will just get the list of available application instances, and join the first one.
 * 
 */
public class GASPClientForBroadcastRMAITest extends UGASPClientHttp implements Runnable {

	private boolean isOwner;
	private String myPassword;
	
	public final static int applicationID = 0;
	
	public final static String OWNER_USER_NAME 					= "T_USR1";
	public final static String OWNER_PASSWORD 					= "T_PWD1";
	public final static String FIRST_OTHER_CLIENT_USER_NAME 	= "T_USR2";
	public final static String FIRST_OTHER_CLIENT_PASSWORD  	= "T_PWD2";
	public final static String SECOND_OTHER_CLIENT_USER_NAME 	= "T_USR3";
	public final static String SECOND_OTHER_CLIENT_PASSWORD  	= "T_PWD3";

	public final static int  MIN_ACTORS = 1;
	public final static int  MAX_ACTORS = 4;
	public final static int  NUMBER_OF_ACTORS_REQUESTED_TO_START_INSTANCE = 3;
	
	private File testFile;
	private String gameLogicTestFilePath;
	
	// if value of this attribute is set to true, broadcast RMAI will be tested also on the game logic side,
	// otherwise, they will be tested only on clients side
	private boolean broadcastWithGameLogic;
	
	private final static String BROADCAST_WITH_GAME_LOGIC_MESSAGE = "Broadcast with server works!";
	

	private GASPClientForBroadcastRMAITest(){
		// set suitable value for RMAI protocol
		getDataTransmissionMgr().getRMAIMgr().setRMAIClientManagerSubclassName(MyRMAIClientManager.class.getName());
	}

	/**
	 * Creates a GASPClientForBroadcastRMAITest that will be the owner of a game instance.
	 * The client created is set to test broadcast without game logic RMAI.
	 * 
	 * @return GASPClientForBroadcastRMAITest the client which creates and start the game instance. 
	 */
	public static GASPClientForBroadcastRMAITest getOwnerToTestBroadcastWithoutGameLogic(){
		GASPClientForBroadcastRMAITest owner = new GASPClientForBroadcastRMAITest();
		owner.isOwner = true;
		owner.getID().setUsername(OWNER_USER_NAME);
		owner.myPassword = OWNER_PASSWORD;
		owner.broadcastWithGameLogic = false;
		return owner;
	}
	
	
	/**
	 * Creates a GASPClientForBroadcastRMAITest that will be the owner of a game instance.
	 * The client created is set to test broadcast with game logic RMAI
	 * 
	 * @param  gameLogicTestFilePath the path of the file where game logic will store its data
	 * @return GASPClientForBroadcastRMAITest the client which creates and start the game instance. 
	 */
	public static GASPClientForBroadcastRMAITest getOwnerToTestBroadcastWithGameLogic(String gameLogicTestFilePath){
		GASPClientForBroadcastRMAITest owner = new GASPClientForBroadcastRMAITest();
		owner.isOwner = true;
		owner.getID().setUsername(OWNER_USER_NAME);
		owner.myPassword = OWNER_PASSWORD;
		owner.broadcastWithGameLogic = true;
		owner.gameLogicTestFilePath = gameLogicTestFilePath;
		return owner;
	}

	/**
	 * Create a GASPClientForBroadcastRMAITest that will try to join a game instance.
	 * 
	 * @return GASPClientForBroadcastRMAITest the client which join the game instance. 
	 */
	public static GASPClientForBroadcastRMAITest getFirstOtherClient(String clientTestFilepath){
		GASPClientForBroadcastRMAITest client = new GASPClientForBroadcastRMAITest();
		client.isOwner = false;
		client.getID().setUsername(FIRST_OTHER_CLIENT_USER_NAME);
		client.myPassword = FIRST_OTHER_CLIENT_PASSWORD;
		client.testFile = new File(clientTestFilepath);
		return client;
	}
	
	
	/**
	 * Create a GASPClientForBroadcastRMAITest that will try to join a game instance.
	 * 
	 * @return GASPClientForBroadcastRMAITest the client which join the game instance. 
	 */
	public static GASPClientForBroadcastRMAITest getSecondOtherClient(String clientTestFilepath){
		GASPClientForBroadcastRMAITest client = new GASPClientForBroadcastRMAITest();
		client.isOwner = false;
		client.getID().setUsername(SECOND_OTHER_CLIENT_USER_NAME);
		client.myPassword = SECOND_OTHER_CLIENT_PASSWORD;
		client.testFile = new File(clientTestFilepath);
		return client;
	}

	
	public File getTestFile(){
		return testFile;
	}
	
	
	public void run(){
		if(isOwner){
			createUGaspApplicationInstance();
			if(getID().isActorSessionIDValid()){
				System.out.println("Application created by owner, waiting for "+(NUMBER_OF_ACTORS_REQUESTED_TO_START_INSTANCE - 1)+" other clients before starting");
			}else{
				System.out.println("GASPClientForBroadcastRMAITest failed in creating a game instance");
				return;
			}
		} else{
			joinUGaspApplicationInstance();
			if(getID().isActorSessionIDValid()){
				System.out.println("Application joined");
			}else{
				System.out.println("GASPClientForBroadcastRMAITest failed in joining a game instance");
				return;
			}
		}
		// for every client, starts the client's routine
		getDataTransmissionMgr().launchDataTransmissionRoutine();
	}

	/*
	 * This method successively proceeds :
	 * - a login to get an sessionID
	 * - an application instance creation which return an actorSessionID.
	 *@return the actorSessionID of the owner
	 */
	private short createUGaspApplicationInstance(){
		// first login with static values of username and password
		getLobbyMgr().firstLogin(applicationID, getID().getUsername(), myPassword);
		if(getID().isActorIDValid()){
			// getUsername() and myPassword are equals respectively to ownerUsername and ownerPassword
			getLobbyMgr().login(getID().getActorID(), getID().getUsername(), myPassword);
			if(getID().isSessionIDValid()){
				getLobbyMgr().createApplicationInstance(getID().getSessionID(), MIN_ACTORS, MAX_ACTORS);
				if(getID().isActorSessionIDValid()){
					return getID().getActorSessionID();	
				} else{
					System.out.println("Failed in creating application instance");
					return -1;
				}
			}else {
				System.out.println("Failed in login");
				return -1;
			}
		}else{
				System.out.println("Failed in first login");
				return -1;
		}
	}

	/*
	 * This method successively proceeds :
	 * - a first login to get an actorID
	 * - a login to get an sessionID
	 * - an application instance creation which return an actorSessionID.
	 *@ return the actorSessionID of the client
	 */
	@SuppressWarnings("rawtypes")
	private short joinUGaspApplicationInstance(){
		getLobbyMgr().firstLogin(applicationID, getID().getUsername(), myPassword);
		if(getID().isActorIDValid()){
			// getUsername() and myPassword are equals respectively to ownerUsername and ownerPassword
			getLobbyMgr().login(getID().getActorID(), getID().getUsername(), myPassword);
			if(getID().isSessionIDValid()){
				// get the list of available game instances
				Vector applicationInstances = getLobbyMgr().getApplicationInstancesInfos(getID().getSessionID());
				if(applicationInstances.size()!=0){
					getLobbyMgr().joinApplicationInstance(getID().getSessionID(), ((ApplicationInstanceInfos)applicationInstances.elementAt(0)).getApplicationInstanceID());
					if(getID().isActorSessionIDValid()){
						return getID().getActorSessionID();	
					} else{
						System.out.println("Failed in creating application instance");
						return -1;
					}
				}else{
					System.out.println("No available application instance");
					return -1;
				}
			}else {
				System.out.println("Failed in login");
				return -1;
			}
		}else{
			System.out.println("Failed in first login");
			return -1;
		}
	}


	/**
	 * Override the onJoinEvent, to implement the required owner behavior for tutorial helloworld.
	 * Event received when a client identified by its actorSessionID and it's username has joined the current instance.
	 * If this event is received by the owner, and if enough actors have joined the instance, owner will start the instance.
	 *
	 *@param	actorSessionID	the actorSessionID of the client
	 *@param username		its username
	 */
	public void onJoinEvent(short actorSessionID, String username) {
		System.out.println("Join event received for actorSessionID = "+actorSessionID+" with pseudo = "+username);
		// Only the owner can start an instance		
		if(isOwner && (getConnectedClientMgr().getNumberOfConnectedClients() == NUMBER_OF_ACTORS_REQUESTED_TO_START_INSTANCE)){
			System.out.println("Enough actors have joined the instance, starting instance");
			// start instance
			getLobbyMgr().startApplicationInstance(getID().getActorSessionID()); 
			if(broadcastWithGameLogic){
				// rmai to prepare tests on the game logic: creation of test file, ... 
				getDataTransmissionMgr().getRMAIMgr().remoteMethodAsynchronousInvocation(MyRMAIClientManager.ActorSessionID.uGASP_GAME_LOGIC.getID(),
													"prepareFileWriting",gameLogicTestFilePath);
				// test on broadcast with game logic RMAI
				getDataTransmissionMgr().getRMAIMgr().remoteMethodAsynchronousInvocation(MyRMAIClientManager.ActorSessionID.BROADCAST_WITH_GAME_LOGIC.getID(),
													"storeBroadcastWithGameLogicMessage",BROADCAST_WITH_GAME_LOGIC_MESSAGE);
			} else{
				// test on broadcast without game logic RMAI 
				getDataTransmissionMgr().getRMAIMgr().remoteMethodAsynchronousInvocation(MyRMAIClientManager.ActorSessionID.BROADCAST_WITHOUT_GAME_LOGIC.getID(),
													"storeBroadcastWithoutGameLogicMessage");
			}
			// rmai to make other client quit tests
			getDataTransmissionMgr().getRMAIMgr().remoteMethodAsynchronousInvocation(MyRMAIClientManager.ActorSessionID.BROADCAST_WITHOUT_GAME_LOGIC.getID(),
												"quitTest");
		}
	}
	

	/**
	 * Method called when a QuitEvent is received from the platform, i.e.
	 * when a client quits the current instance.
	 * 
	 * @param actorSessionID the actorSessionID of the client which raised the event
	 */
	public void onQuitEvent(short actorSessionID) {
		System.out.print("Quit event received from actorSessionID = "+actorSessionID);
		if(isOwner && getConnectedClientMgr().getNumberOfConnectedClients() == 1){
			System.out.println(" --> Owner is the last connected client, instance can be quit.");
			getLobbyMgr().quitApplicationInstance(getID().getActorSessionID());
		}else{
			System.out.println(" --> "+getConnectedClientMgr().getNumberOfConnectedClients()+" are still connected.");
		}
	}
}
