package org.ugasp.client.test;


import java.io.DataOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import org.ugasp.client.http.UGASPClientHttp;



/**
 * This class is used to test the whole RMAI process on uGASP.
 * Depending on the value of "clientToGameLogicRMAI" attribute, client has different behaviors: 
 * <ul>
 * <li>if "clientToGameLogicRMAI" is set to true, client invokes several methods on the game logic, write parameters of this invoked method in a file,
 * and compare parameters written with those received by game logic (also written in a file)</li>
 * <li>if "clientToGameLogicRMAI" is set to false, client ask the game logic to invoke methods on it, write the parameters of methods invoked on the client side,
 * and compare them with those sent by game logic (also written in a file)</li>
 * </ul> 
 * 
 **/
public class GASPClientForUnicastRMAITests extends UGASPClientHttp implements Runnable {

	
	public final static int applicationID = 0;
	public final static String username = "TEST_USR";
	public final static String password = "TEST_PWD";
	public final static String RMAI_SERIALIZABLE_OBJECTS_FOLDER_NAME = "org.ugasp.client.test";
	public final static int  MIN_ACTORS = 1;
	public final static int  MAX_ACTORS = 4;
	
	private File testFile;
	private String gameLogicTestFilePath;
	
	
	// if value of this attribute is set to true, RMAI will be tested from client to game logic,
	// otherwise, they will be tested from game logic to client
	private boolean clientToGameLogicRMAI;
	
	private final static String STRING_TO_TEST_MooDS = "MooDS works!";
	private final static MyRMAIObject rmaiSerializable0 = initRMAISerializable0();
	private final static MyRMAIObject rmaiSerializable1 = initRMAISerializable1();

	
	private GASPClientForUnicastRMAITests(String clientTestFilepath, String gameLogicTestFilePath, boolean clientToGameLogicRMAI){
		this.clientToGameLogicRMAI = clientToGameLogicRMAI;
		this.gameLogicTestFilePath = gameLogicTestFilePath;
		testFile = new File(clientTestFilepath);
		// set suitable values for RMAI protocol
		getDataTransmissionMgr().getRMAIMgr().setRMAIClientManagerSubclassName(MyRMAIClientManager.class.getName());
		getDataTransmissionMgr().getRMAIMgr().setRMAISerializableObjectsFolder(RMAI_SERIALIZABLE_OBJECTS_FOLDER_NAME);
		// and for MooDS protocol
		getDataTransmissionMgr().setCustomTypes(new CustomTypesImpl());
	}
	
	public static GASPClientForUnicastRMAITests createClient(String clientTestFilepath, String gameLogicTestFilePath, boolean clientToGameLogicRMAI){
		return new GASPClientForUnicastRMAITests(clientTestFilepath, gameLogicTestFilePath, clientToGameLogicRMAI);
	}
	
	private final static MyRMAIObject initRMAISerializable0(){
		return new MyRMAIObject("Bodmer", "PSG", Short.valueOf((short)25), Byte.valueOf((byte)8));		
	}
	
	private final static MyRMAIObject initRMAISerializable1(){
		return new MyRMAIObject("Sakho", "PSG", Short.valueOf((short)21), Byte.valueOf((byte)4));
	}
	
	public File getTestFile(){
		return testFile;
	}

	public void run(){
		boolean applicationInstanceStarted = createFirstUGaspApplicationInstance();
		if(applicationInstanceStarted){
			System.out.println("Application started, launching data transmission routine");
			getDataTransmissionMgr().launchDataTransmissionRoutine();
		}else{
			System.out.println("GaspClientForTest login procedure failed");
			return;
		}
	}

	/*
	 * This method successively proceeds :
	 * - a first login to get an actorID
	 * - a login to get an sessionID
	 * - an application instance creation which return an actorSessionID
	 * - start of the application instance corresponding to actorSessionID.
	 */
	private boolean createFirstUGaspApplicationInstance(){
		// the result of the first login is saved into myActorID
		getLobbyMgr().firstLogin(applicationID, username, password);
		if(getID().isActorIDValid()){
			// the result of the login is saved into mySessionID
			getLobbyMgr().login(getID().getActorID(), username, password);
			if(getID().isSessionIDValid()){
				// the result of the login is saved into myActorSessionID
				getLobbyMgr().createApplicationInstance(getID().getSessionID(), MIN_ACTORS, MAX_ACTORS);
				if(getID().isActorSessionIDValid()){
					return getLobbyMgr().startApplicationInstance(getID().getActorSessionID());
				} else{
					System.out.println("Failed in creating application instance. \n" +
							"If your application in this lifecycle mode = \"manual\", it could be because another client " +
							"was launching the test process exactly in the same time than you.\n");
					return false;
				}
			}else {
				System.out.println("Failed in login");
				return false;
			}
		} else{
			System.out.println("Failed in first login");
			return false;
		}
	}

	@Override
	/**
	 * Behavior of test client is to send several representative values of each types handled
	 * by RMAI, and to write those values in a file, to allow a comparison with values received
	 * by the game logic.
	 */
	public void onJoinEvent(short actorSessionID, String username) {
		// on the receiving its own joinEvent
		if(actorSessionID == getID().getActorSessionID()){
			// rmai to prepare tests: creation of test file, ...  
			getDataTransmissionMgr().getRMAIMgr().remoteMethodAsynchronousInvocation(MyRMAIClientManager.ActorSessionID.uGASP_GAME_LOGIC.getID(),
					"prepareFileWriting",
					gameLogicTestFilePath);
			System.out.println("RMAI prepare file writing OK");
			// RMAI from client to game logic
			if(clientToGameLogicRMAI){
				//execute RMAI
				clientToGameLogicRMAI();
				// send MooDS message
				clientToGameLogicMooDS();
				//rmai to quit tests
				getDataTransmissionMgr().getRMAIMgr().remoteMethodAsynchronousInvocation(MyRMAIClientManager.ActorSessionID.uGASP_GAME_LOGIC.getID(),
						"askToQuitTest");
				// store data
				storeDataInTestFile();
			}// RMAI from game logic to client
			else{
				getDataTransmissionMgr().getRMAIMgr().remoteMethodAsynchronousInvocation(MyRMAIClientManager.ActorSessionID.uGASP_GAME_LOGIC.getID(),
						"askToCreateDataOutputStream");
			}
		}
	}
	
	/*
	 * Contains several RMAI tested from client to game logic.
	 */
	private void clientToGameLogicRMAI(){
		// rmai to test bytes
		getDataTransmissionMgr().getRMAIMgr().remoteMethodAsynchronousInvocation(MyRMAIClientManager.ActorSessionID.uGASP_GAME_LOGIC.getID(),
				"testBytes",
				Byte.MIN_VALUE,new Byte((byte)0),Byte.MAX_VALUE);
		// rmai to test shorts
		getDataTransmissionMgr().getRMAIMgr().remoteMethodAsynchronousInvocation(MyRMAIClientManager.ActorSessionID.uGASP_GAME_LOGIC.getID(),
				"testShorts",
				Short.MIN_VALUE,new Short((short)0), Short.MAX_VALUE);
		// rmai to test integers
		getDataTransmissionMgr().getRMAIMgr().remoteMethodAsynchronousInvocation(MyRMAIClientManager.ActorSessionID.uGASP_GAME_LOGIC.getID(),
				"testIntegers",
				Integer.MIN_VALUE,new Integer((int)0), Integer.MAX_VALUE);
		// rmai to test longs
		getDataTransmissionMgr().getRMAIMgr().remoteMethodAsynchronousInvocation(MyRMAIClientManager.ActorSessionID.uGASP_GAME_LOGIC.getID(),
				"testLongs",
				Long.MIN_VALUE,new Long((long)0), Long.MAX_VALUE);
		// rmai to test floats
		getDataTransmissionMgr().getRMAIMgr().remoteMethodAsynchronousInvocation(MyRMAIClientManager.ActorSessionID.uGASP_GAME_LOGIC.getID(),
				"testFloats",
				Float.NEGATIVE_INFINITY,Float.MIN_VALUE,
				new Float((float)0), Float.NaN,
				Float.MAX_VALUE,Float.POSITIVE_INFINITY);
		// rmai to test doubles
		getDataTransmissionMgr().getRMAIMgr().remoteMethodAsynchronousInvocation(MyRMAIClientManager.ActorSessionID.uGASP_GAME_LOGIC.getID(),
				"testDoubles",
				Double.NEGATIVE_INFINITY,Double.MIN_VALUE,
				new Double((double)0), Double.NaN,
				Double.MAX_VALUE,Double.POSITIVE_INFINITY);
		// rmai to test booleans
		getDataTransmissionMgr().getRMAIMgr().remoteMethodAsynchronousInvocation(MyRMAIClientManager.ActorSessionID.uGASP_GAME_LOGIC.getID(),
				"testBooleans",
				Boolean.TRUE,Boolean.FALSE);
		// rmai to test Strings	
		getDataTransmissionMgr().getRMAIMgr().remoteMethodAsynchronousInvocation(MyRMAIClientManager.ActorSessionID.uGASP_GAME_LOGIC.getID(),
				"testStrings",
				"AZERTY","azerty");
		// rmai to test RMAISerializable	
		getDataTransmissionMgr().getRMAIMgr().remoteMethodAsynchronousInvocation(MyRMAIClientManager.ActorSessionID.uGASP_GAME_LOGIC.getID(),
				"testSerializables",rmaiSerializable0,rmaiSerializable1);
	}
	
	private void clientToGameLogicMooDS(){
		FirstMooDSObject fmo = new FirstMooDSObject();
		fmo.setText(STRING_TO_TEST_MooDS);
		getDataTransmissionMgr().sendMooDSObject(fmo);
	}

	/*
	 * This method store the data sent to game logic in a file to allow a later comparison
	 * with data received by game logic.
	 */
	private void storeDataInTestFile(){
		try{
			// get stream from file 
			FileOutputStream fileOutputStream = new FileOutputStream(testFile);
			DataOutputStream dataOutputStream = new DataOutputStream(fileOutputStream);
			// writing values on file for later comparison
			// bytes
			dataOutputStream.writeByte(Byte.MIN_VALUE);
			dataOutputStream.writeByte(new Byte((byte)0));
			dataOutputStream.writeByte(Byte.MAX_VALUE);
			// shorts
			dataOutputStream.writeShort(Short.MIN_VALUE);
			dataOutputStream.writeShort(new Short((short)0));
			dataOutputStream.writeShort(Short.MAX_VALUE);
			// integers
			dataOutputStream.writeInt(Integer.MIN_VALUE);
			dataOutputStream.writeInt(new Integer(0));
			dataOutputStream.writeInt(Integer.MAX_VALUE);
			// longs
			dataOutputStream.writeLong(Long.MIN_VALUE);
			dataOutputStream.writeLong(new Long((long)0));
			dataOutputStream.writeLong(Long.MAX_VALUE);
			// floats
			dataOutputStream.writeFloat(Float.NEGATIVE_INFINITY);
			dataOutputStream.writeFloat(Float.MIN_VALUE);
			dataOutputStream.writeFloat(new Float((float)0));
			dataOutputStream.writeFloat(Float.NaN);
			dataOutputStream.writeFloat(Float.MAX_VALUE);
			dataOutputStream.writeFloat(Float.POSITIVE_INFINITY);
			// doubles
			dataOutputStream.writeDouble(Double.NEGATIVE_INFINITY);
			dataOutputStream.writeDouble(Double.MIN_VALUE);
			dataOutputStream.writeDouble(new Double((double)0));
			dataOutputStream.writeDouble(Double.NaN);
			dataOutputStream.writeDouble(Double.MAX_VALUE);
			dataOutputStream.writeDouble(Double.POSITIVE_INFINITY);
			// booleans
			dataOutputStream.writeBoolean(Boolean.TRUE);
			dataOutputStream.writeBoolean(Boolean.FALSE);
			// Strings
			dataOutputStream.writeUTF("AZERTY");
			dataOutputStream.writeUTF("azerty");
			// RMAISerializables
			dataOutputStream.writeUTF(rmaiSerializable0.toString());
			dataOutputStream.writeUTF(rmaiSerializable1.toString());			
			// MooDS
			dataOutputStream.writeUTF(STRING_TO_TEST_MooDS);
			// close streams
			dataOutputStream.flush();
			dataOutputStream.close();    
		}catch(FileNotFoundException e){
			e.printStackTrace();
		}catch(IOException e){
			e.printStackTrace();
		}       
	}
	
	public void treatMooDSMessage(Object moodsObject,short senderActorSessionID){
		if(moodsObject instanceof FirstMooDSObject){
			FirstMooDSObject fmo = (FirstMooDSObject)moodsObject;
			// store message on the DataOutputStream used to store RMAI
			try{
				MyRMAIClientManager.storeMooDSMessage(fmo.getText());
			}catch(IOException e){
				e.printStackTrace();
			}
		}
	}
}
