package org.ugasp.client.test;


import java.io.DataOutputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import org.ugasp.client.RMAIClientManager;



/**
 * This class holds the methods that will be invoked remotely by other clients 
 * or by game logic.
 * 
 * Declare methods respecting following properties:
 * <ul>
 * <li>they must be instances methods, and not static ones</li>
 * <li>types handled as parameters are Byte, Short, Integer, Long, Float, Double, Boolean, String, Date and Calendar</li>
 * <li>as method invocations through RMAI protocol are asynchronous,
 * only void methods can be invoked. However a callback mechanism is provided, allowing to invoke remote method
 * on the entity which did the invocation first (as it is done in replyToMessage(String message))</li>
 * </ul>
 * 
 * Entity which has invoked a method located in this class can be retrieved, using getSenderActorSessionID().
 * 
 * @see replyToMessage(String message)
 **/


public class MyRMAIClientManager extends RMAIClientManager{

	private static DataOutputStream clientTestFileDos = null;
	private final static String BROADCAST_WITHOUT_GAME_LOGIC_MESSAGE = "Broadcast without server works!";
	
	
	/*********************************************************************************************************/
	/********************************    Method used to test MooDS    ****************************************/
	/*********************************************************************************************************/

	
	public static void storeMooDSMessage(String mooDSMessage) throws IOException{
		if(clientTestFileDos != null){
			// writing value on file for later comparison
			clientTestFileDos.writeUTF(mooDSMessage);
		}
	}
	
	
	/*********************************************************************************************************/
	/*************************************    Remote Methods    **********************************************/
	/*********************************************************************************************************/
	
	public void quitTest(){
		getClient().getLobbyMgr().quitApplicationInstance(getClient().getID().getActorSessionID());
	}
	
	/******************************************************************************************************/
	/**************************  Method used to test unicast RMAI on the game logic side ******************/
	/******************************************************************************************************/
	
	/**
	 * This method initializes the data output stream used for test.
	 * Then, it ask game logic to start tests, via a RMAI to the gameLogicToClientTests method,
	 * located on the game logic side.
	 */
	public void createDataOutputStream(){
		try {
			FileOutputStream fos = new FileOutputStream(((GASPClientForUnicastRMAITests)getClient()).getTestFile());
			clientTestFileDos = new DataOutputStream(fos);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		// as soon as data output stream is created, game logic can launch tests
		getClient().getDataTransmissionMgr().getRMAIMgr().remoteMethodAsynchronousInvocation(getSenderActorSessionID(), "gameLogicToClientTests");
	}
	
	
	public void testBytes(Byte b0, Byte b1, Byte b2) throws IOException{
		// writing values on file for later comparison
		clientTestFileDos.writeByte(b0);
		clientTestFileDos.writeByte(b1);
		clientTestFileDos.writeByte(b2);
	}
	
	
	public void testShorts(Short s0, Short s1, Short s2) throws IOException{
		// writing values on file for later comparison
		clientTestFileDos.writeShort(s0);
		clientTestFileDos.writeShort(s1);
		clientTestFileDos.writeShort(s2);
	}
	
	
	public void testIntegers(Integer i0, Integer i1, Integer i2) throws IOException{
		// writing values on file for later comparison
		clientTestFileDos.writeInt(i0);
		clientTestFileDos.writeInt(i1);
		clientTestFileDos.writeInt(i2);
	}
	
	
	public void testLongs(Long l0, Long l1, Long l2) throws IOException{
		// writing values on file for later comparison
		clientTestFileDos.writeLong(l0);
		clientTestFileDos.writeLong(l1);
		clientTestFileDos.writeLong(l2);
	}
	
	
	public void testFloats(Float f0, Float f1, Float f2, Float f3, Float f4, Float f5) throws IOException{
		// writing values on file for later comparison
		clientTestFileDos.writeFloat(f0);
		clientTestFileDos.writeFloat(f1);
		clientTestFileDos.writeFloat(f2);
		clientTestFileDos.writeFloat(f3);
		clientTestFileDos.writeFloat(f4);
		clientTestFileDos.writeFloat(f5);
	}
	
	
	public void testDoubles(Double d0, Double d1, Double d2, Double d3, Double d4, Double d5) throws IOException{
		// writing values on file for later comparison
		clientTestFileDos.writeDouble(d0);
		clientTestFileDos.writeDouble(d1);
		clientTestFileDos.writeDouble(d2);
		clientTestFileDos.writeDouble(d3);
		clientTestFileDos.writeDouble(d4);
		clientTestFileDos.writeDouble(d5);
	}
	
	
	public void testBooleans(Boolean b0, Boolean b1) throws IOException{
		// writing values on file for later comparison
		clientTestFileDos.writeBoolean(b0);
		clientTestFileDos.writeBoolean(b1);
	}
	
	
	public void testStrings(String s0, String s1) throws IOException{
		// writing values on file for later comparison
		clientTestFileDos.writeUTF(s0);
		clientTestFileDos.writeUTF(s1);
	}
	
	public void testSerializables(MyRMAIObject fro1,MyRMAIObject fro2) throws IOException{
		// writing String representations of FirstRMAIObjects for later comparison
		clientTestFileDos.writeUTF(fro1.toString());
		clientTestFileDos.writeUTF(fro2.toString());
	}
	
	/******************************************************************************************************/
	/**************************  Method used to test broadcast RMAI on the game logic side ******************/
	/******************************************************************************************************/

	
	public void storeBroadcastWithGameLogicMessage(String message){
		if(((GASPClientForBroadcastRMAITest)getClient()).getTestFile() != null){
			try {
				FileOutputStream fos = new FileOutputStream(((GASPClientForBroadcastRMAITest)getClient()).getTestFile());
				clientTestFileDos = new DataOutputStream(fos);
				clientTestFileDos.writeUTF(message);
				clientTestFileDos.close();
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		} else{
			System.out.println("Client Test File is NULL, can't write on it");
		}
	}
	
	
	public void storeBroadcastWithoutGameLogicMessage(){
		if(((GASPClientForBroadcastRMAITest)getClient()).getTestFile() != null){
			try {
				FileOutputStream fos = new FileOutputStream(((GASPClientForBroadcastRMAITest)getClient()).getTestFile());
				clientTestFileDos = new DataOutputStream(fos);
				clientTestFileDos.writeUTF(BROADCAST_WITHOUT_GAME_LOGIC_MESSAGE);
				clientTestFileDos.close();
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		} else{
			System.out.println("Client Test File is NULL, can't write on it");
		}
		
	}
}
