package org.ugasp.demonstrator.client.protocol.rmai;


import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.UUID;

import org.ugasp.client.test.GASPClientForBroadcastRMAITest;

import org.junit.*;
import static org.junit.Assert.*;


/**
 * The goals of following tests is to check validity of whole RMAI process, 
 * on the broadcast of invocation on clients and on game logic.
 * The test proceed this way:
 * <ul>
 * <li>a client - the owner - log and create an application instance</li>
 * <li>two other clients log and join the instance created by the owner</li>
 * <li>the owner start the instance when the two clients have joined the instance</li>
 * <li>then it send a broadcast RMAI on the two other clients and on the game logic</li>
 * <li>on the reception of this invocation, clients and game logic write the message received by the owner in a file</li>
 * <li>still on the reception of this invocation, clients quit the instance</li>
 * <li>then, when the owner is the last client connected to the instance, it quit (and hence end) the instance too</li>
 * <li>finally, message written in clients and game logic files are compared</li>
 * </ul>
 *
 */
public class TestBroadcastWithGameLogicRMAI{

	private final static String RMAI_TESTS_FILES_FOLDER_PATH 		= System.getProperty("user.dir")+File.separator+"target"+File.separator+"rmai-test-files"+File.separator ;
	private final static String FIRST_OTHER_CLIENT_TEST_FILE_NAME 	= "firstOtherClientTestFile";
	private final static String SECOND_OTHER_CLIENT_TEST_FILE_NAME 	= "secondOtherClientTestFile";
	private final static String GAME_LOGIC_TEST_FILE_NAME			= "gameLogicTestFile"; 	
	private static int tempID;
	
	private static String firstOtherClientTestFilePath;
	private static String secondOtherClientTestFilePath;
	private static String gameLogicTestFilePath;
	
	private static File firstOtherClientTestFile;
	private static File secondOtherClientTestFile;
	private static File gameLogicTestFile;
	
	private static DataInputStream firstOtherClientTestFileDis = null;
	private static DataInputStream secondOtherClientTestFileDis = null;
	private static DataInputStream gameLogicTestFileDis = null;
	
	private final static long delay = 1000;
	
	public TestBroadcastWithGameLogicRMAI(){}
	
	/******************************************************************************************/
	/************************************ Tests preparations **********************************/
	/******************************************************************************************/
	@BeforeClass
	public static void setUpTest() {
		System.out.println("\n>>>>>>  Tests on BROADCAST with GAME LOGIC RMAI <<<<<<");
		// Creation of folder used to store clients and game logic files
		File folder = new File(RMAI_TESTS_FILES_FOLDER_PATH);
		if(!folder.exists()){
			folder.mkdirs();
		}
		// ID generation
		generateTempID();
		// Creating file paths
		firstOtherClientTestFilePath 	= RMAI_TESTS_FILES_FOLDER_PATH+FIRST_OTHER_CLIENT_TEST_FILE_NAME+getTempID();
		secondOtherClientTestFilePath 	= RMAI_TESTS_FILES_FOLDER_PATH+SECOND_OTHER_CLIENT_TEST_FILE_NAME+getTempID();
		gameLogicTestFilePath 			= RMAI_TESTS_FILES_FOLDER_PATH+GAME_LOGIC_TEST_FILE_NAME+getTempID();
		// launching clients
		try {
			launchClients();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		// preparation to test
		prepareFilesReading();
	}

	@AfterClass
	public static void tearDownTest() throws IOException{
		// close input streams
		if(firstOtherClientTestFileDis != null)	firstOtherClientTestFileDis.close();
		if(secondOtherClientTestFileDis!= null)	secondOtherClientTestFileDis.close();
		if(gameLogicTestFileDis != null)		gameLogicTestFileDis.close();
		// delete files (folder isn't deleted in case where another client launch concurrently the test procedure
		firstOtherClientTestFile.delete();
		secondOtherClientTestFile.delete();
		gameLogicTestFile.delete();
		System.out.println(">>>>>>  Tests on BROADCAST with GAME LOGIC RMAI finished <<<<<<\n");
	}


	private static void launchClients() throws InterruptedException{
		// creating 3 clients
		// owner is set to test broadcast with game logic
		GASPClientForBroadcastRMAITest ownerClient 		= GASPClientForBroadcastRMAITest.getOwnerToTestBroadcastWithGameLogic(gameLogicTestFilePath);
		GASPClientForBroadcastRMAITest firstOtherClient = GASPClientForBroadcastRMAITest.getFirstOtherClient(firstOtherClientTestFilePath);
		GASPClientForBroadcastRMAITest secondOtherClient= GASPClientForBroadcastRMAITest.getSecondOtherClient(secondOtherClientTestFilePath);
		// launching clients
		Thread ownerThread 			= new Thread(ownerClient);
		Thread firstClientThread 	= new Thread(firstOtherClient);
		Thread secondClientThread 	= new Thread(secondOtherClient);
		ownerThread.start();
		Thread.sleep(delay);
		firstClientThread.start();
		Thread.sleep(delay);
		secondClientThread.start();
		Thread.sleep(delay);
		// wait until every client has stopped 
		while(ownerThread.isAlive() || firstClientThread.isAlive() || secondClientThread.isAlive()){}
	}


	private static void prepareFilesReading(){
		firstOtherClientTestFile 	= new File(firstOtherClientTestFilePath);
		secondOtherClientTestFile 	= new File(secondOtherClientTestFilePath);
		gameLogicTestFile 			= new File(gameLogicTestFilePath);
		try{
			// get stream from first other client test file
			FileInputStream firstOtherClientTestFileInputStream = new FileInputStream(firstOtherClientTestFile);
			firstOtherClientTestFileDis = new DataInputStream(firstOtherClientTestFileInputStream);
			// get stream from second other client test file
			FileInputStream secondOtherClientTestFileInputStream = new FileInputStream(secondOtherClientTestFile);
			secondOtherClientTestFileDis = new DataInputStream(secondOtherClientTestFileInputStream);
			// get stream from game logic test file 
			FileInputStream gameLogicTestFileInputStream = new FileInputStream(gameLogicTestFile);
			gameLogicTestFileDis = new DataInputStream(gameLogicTestFileInputStream);
		}catch(FileNotFoundException e){
			e.printStackTrace();
		}
	}
	
	
	private static void generateTempID(){
		// generate a random UUID
		UUID uuid =  UUID.randomUUID();
		// get a significant positive int from this UUID
		int id = Math.abs(uuid.hashCode());
		tempID = id;
	}
	
	
	private static int getTempID(){
		return tempID;
	}

	/******************************************************************************************/
	/************************************ Tests Methods ****************************************/
	/******************************************************************************************/
	
	@org.junit.Test
	public void testBroadcastWithGameLogic()throws IOException{
		String firstOtherClientMessage 	= firstOtherClientTestFileDis.readUTF();
		String secondOtherClientMessage = secondOtherClientTestFileDis.readUTF();
		String gameLogicMessage 		= gameLogicTestFileDis.readUTF();
		assertEquals(firstOtherClientMessage,secondOtherClientMessage);
		assertEquals(firstOtherClientMessage,gameLogicMessage);
	}
	
}

