package org.ugasp.demonstrator.client.protocol.rmai;


import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.UUID;

import org.ugasp.client.test.GASPClientForUnicastRMAITests;

import org.junit.*;
import static org.junit.Assert.*;


/**
 * The goals of following tests is to check validity of whole data transmission process, 
 * from the client to the game logic, i.e.:
 * <ul>
 * <li>client login to test game logic</li>
 * <li>creation of an application instance by the client</li>
 * <li>invocation of several method on the game logic side, with every types handled by RMAI</li>
 * <li>sending of a FirstMooDSMessage, containing a simple string based message</li>
 * <li>comparison of parameters "sent" by the client with those "received" by game logic</li>
 * </ul>
 *
 */
public class TestClientToGameLogicRMAI{

	private final static String RMAI_TESTS_FILES_FOLDER_PATH = System.getProperty("user.dir")+File.separator+"target"+File.separator+"rmai-test-files"+File.separator ;
	private final static String CLIENT_TEST_FILE_NAME 		= "clientTestFile"; 
	private final static String GAME_LOGIC_TEST_FILE_NAME	= "gameLogicTestFile"; 	
	private static int tempID;
	
	private static String clientTestFilePath;
	private static String gameLogicTestFilePath;
	
	private static File clientTestFile;
	private static File gameLogicTestFile;
	
	private static DataInputStream clientTestFileDis = null;
	private static DataInputStream gameLogicTestFileDis = null;
	
	public TestClientToGameLogicRMAI(){}

	/******************************************************************************************/
	/************************************ Tests preparations **********************************/
	/******************************************************************************************/
	
	@BeforeClass
	public static void setUpTest() {
		System.out.println("\n>>>>>>  Tests on CLIENT to GAME LOGIC RMAI <<<<<<");
		// Creation of folder used to store clients and game logic files
		File folder = new File(RMAI_TESTS_FILES_FOLDER_PATH);
		if(!folder.exists()){
			folder.mkdirs();
		}
		// ID generation
		generateTempID();
		// Creating file paths
		clientTestFilePath = RMAI_TESTS_FILES_FOLDER_PATH+CLIENT_TEST_FILE_NAME+getTempID();
		gameLogicTestFilePath = RMAI_TESTS_FILES_FOLDER_PATH+GAME_LOGIC_TEST_FILE_NAME+getTempID();
		// launching client
		launchClient();
		// preparation to test
		prepareFilesReading();
	}

	@AfterClass
	public static void tearDownTest() throws IOException{
		// close input streams
		if(clientTestFileDis != null)clientTestFileDis.close();
		if(gameLogicTestFileDis != null)gameLogicTestFileDis.close();
		// delete files (folder isn't deleted in case where another client launch concurrently the test procedure
		clientTestFile.delete();
		gameLogicTestFile.delete();
		System.out.println(">>>>>>  Tests on CLIENT to GAME LOGIC RMAI finished <<<<<<\n");
	}


	private static void launchClient(){
		// client to test RMAI from client to game logic
		GASPClientForUnicastRMAITests testGaspClient = GASPClientForUnicastRMAITests.createClient(clientTestFilePath,gameLogicTestFilePath,true);
		Thread m_thread = new Thread(testGaspClient);
		m_thread.start();
		// wait until Test client has stopped (i.e. when server receives properly its RMAI) 
		while(m_thread.isAlive()){}
	}


	private static void prepareFilesReading(){
		clientTestFile = new File(clientTestFilePath);
		gameLogicTestFile = new File(gameLogicTestFilePath);
		try{
			// get stream from client test file
			FileInputStream clientTestFileInputStream = new FileInputStream(clientTestFile);
			clientTestFileDis = new DataInputStream(clientTestFileInputStream);
			// get stream from game logic test file 
			FileInputStream gameLogicTestFileInputStream = new FileInputStream(gameLogicTestFile);
			gameLogicTestFileDis = new DataInputStream(gameLogicTestFileInputStream);
		}catch(FileNotFoundException e){
			e.printStackTrace();
		}
	}
	
	
	private static void generateTempID(){
		// generate a random UUID
		UUID uuid =  UUID.randomUUID();
		// get a significant positive int from this UUID
		int id = Math.abs(uuid.hashCode());
		tempID = id;
	}
	
	
	private static int getTempID(){
		return tempID;
	}

	/******************************************************************************************/
	/************************************ Tests Methods ****************************************/
	/******************************************************************************************/
	
	@org.junit.Test
	public void testBytes()throws IOException{
		assertEquals(clientTestFileDis.readByte(),gameLogicTestFileDis.readByte());
		assertEquals(clientTestFileDis.readByte(),gameLogicTestFileDis.readByte());
		assertEquals(clientTestFileDis.readByte(),gameLogicTestFileDis.readByte());
	}
	
	
	@org.junit.Test
	public void testShorts()throws IOException{
		assertEquals(clientTestFileDis.readShort(),gameLogicTestFileDis.readShort());
		assertEquals(clientTestFileDis.readShort(),gameLogicTestFileDis.readShort());
		assertEquals(clientTestFileDis.readShort(),gameLogicTestFileDis.readShort());
	}

	@org.junit.Test
	public void testIntegers()throws IOException{
		assertEquals(clientTestFileDis.readInt(),gameLogicTestFileDis.readInt());
		assertEquals(clientTestFileDis.readInt(),gameLogicTestFileDis.readInt());
		assertEquals(clientTestFileDis.readInt(),gameLogicTestFileDis.readInt());
	}

	@org.junit.Test
	public void testLongs()throws IOException{
		assertEquals(clientTestFileDis.readLong(),gameLogicTestFileDis.readLong());
		assertEquals(clientTestFileDis.readLong(),gameLogicTestFileDis.readLong());
		assertEquals(clientTestFileDis.readLong(),gameLogicTestFileDis.readLong());
	}

	@org.junit.Test
	public void testFloats()throws IOException{
		assertEquals(clientTestFileDis.readFloat(),gameLogicTestFileDis.readFloat());
		assertEquals(clientTestFileDis.readFloat(),gameLogicTestFileDis.readFloat());
		assertEquals(clientTestFileDis.readFloat(),gameLogicTestFileDis.readFloat());
		assertEquals(clientTestFileDis.readFloat(),gameLogicTestFileDis.readFloat());
		assertEquals(clientTestFileDis.readFloat(),gameLogicTestFileDis.readFloat());
		assertEquals(clientTestFileDis.readFloat(),gameLogicTestFileDis.readFloat());
	}
	
	@org.junit.Test
	public void testDoubles()throws IOException{
		assertEquals(clientTestFileDis.readDouble(),gameLogicTestFileDis.readDouble());
		assertEquals(clientTestFileDis.readDouble(),gameLogicTestFileDis.readDouble());
		assertEquals(clientTestFileDis.readDouble(),gameLogicTestFileDis.readDouble());
		assertEquals(clientTestFileDis.readDouble(),gameLogicTestFileDis.readDouble());
		assertEquals(clientTestFileDis.readDouble(),gameLogicTestFileDis.readDouble());
		assertEquals(clientTestFileDis.readDouble(),gameLogicTestFileDis.readDouble());
	}

	@org.junit.Test
	public void testBooleans()throws IOException{
		assertEquals(clientTestFileDis.readBoolean(),gameLogicTestFileDis.readBoolean());
		assertEquals(clientTestFileDis.readBoolean(),gameLogicTestFileDis.readBoolean());
	}
	
	@org.junit.Test
	public void testStrings()throws IOException{
		assertEquals(clientTestFileDis.readUTF(),gameLogicTestFileDis.readUTF());
		assertEquals(clientTestFileDis.readUTF(),gameLogicTestFileDis.readUTF());
	}
	
	@org.junit.Test
	public void testSerializables()throws IOException{
		assertEquals(clientTestFileDis.readUTF(),gameLogicTestFileDis.readUTF());
		assertEquals(clientTestFileDis.readUTF(),gameLogicTestFileDis.readUTF());
	}
	
	@org.junit.Test
	public void testMooDS()throws IOException{
		assertEquals(clientTestFileDis.readUTF(),gameLogicTestFileDis.readUTF());
	}
}

