package org.ugasp.demonstrator.client.protocol.rmai;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.ugasp.client.datatransmission.protocol.rmai.tools.InvokedMethod;
import org.ugasp.client.datatransmission.protocol.rmai.tools.RMAIUtils;
import org.ugasp.client.test.MyRMAIObject;

import junit.framework.TestCase;

/**
 * Unit tests for RMAIUtils class.
 * Following methods are made to test:
 * <ul>
 * <li>serialization/deserialization process</li>
 * <li>method invocation process</li>
 * <li>whole RMAI process (serialization/deserialization + invocation)</li>
 * </ul>
 * on all types handled by RMAI.
 * 
 */
public class TestRMAIUtils extends TestCase
{
	private Object[] params = null;
	private final static String DEFAULT_METHOD_NAME 	= 	"defaultMethodName";
	
	private final static RMAIUtils rmaiUtils 			= 	new RMAIUtils();
	
	private final static Object[] byteValues 			= 	{Byte.MIN_VALUE, new Byte((byte)0), Byte.MAX_VALUE};
	private final static Object[] shortValues 			= 	{Short.MIN_VALUE, new Short((short)0), Short.MAX_VALUE};
	private final static Object[] integerValues 		= 	{Integer.MIN_VALUE, new Integer(0), Integer.MAX_VALUE};
	private final static Object[] longValues 			= 	{Long.MIN_VALUE, new Long(0), Long.MAX_VALUE};
	private final static Object[] floatValues 			= 	{Float.NEGATIVE_INFINITY, Float.MIN_VALUE, 
															new Float(0), Float.NaN, 
															Float.MAX_VALUE, Float.POSITIVE_INFINITY};
	private final static Object[] doubleValues 			= 	{Double.NEGATIVE_INFINITY, Double.MIN_VALUE, 
															new Double(0), Double.NaN, 
															Double.MAX_VALUE, Double.POSITIVE_INFINITY};
	private final static Object[] booleanValues 		= 	{Boolean.TRUE, Boolean.FALSE};
	// special "character" should be tested to provide informations about managed encoding : 
	// UTF-16(encoding used by java for String and character, or UTF-8(used by DataStreams)?
	private final static Object[] stringValues 			= 	{"AZERTY","azerty"/*," ","?","�","�"*/};
	private final static Object[] serializableValues	=	{new MyRMAIObject("Chantome","PSG",Short.valueOf((short)23),Byte.valueOf((byte)10)),
															new MyRMAIObject("Luyindula","PSG",Short.valueOf((short)29),Byte.valueOf((byte)12))};
	private final static Object[] arrayOfObjectsValues	=	{byteValues,shortValues,integerValues,
															longValues,floatValues,doubleValues,
															booleanValues,stringValues};
	
	/**
	 * Create the test case
	 *
	 * @param testName name of the test case
	 */
	public TestRMAIUtils(String testName){
		super(testName);
	}
	
	
	/********************************** Tests on Bytes ********************************/
	
	/**
	 * Test serialization/deserialization process for Bytes values.
	 * Byte values contained in an Object[] array, and a method name are serialized to a String.
	 * Then, an InvokedMethod is create from parsing of the serialized string.
	 * Eventually, initial method name and parameters are compared with retrieved ones.
	 * 
	 */
	public void testBytesSerialization() throws Exception, ClassNotFoundException,IllegalAccessException,InstantiationException{
		// serialize method name and param
		String invokedMethodXMLSerialized = rmaiUtils.methodInvocation(DEFAULT_METHOD_NAME, byteValues);
		// deserialize the XML into an InvokeMethod object
		InvokedMethod toInvoke = rmaiUtils.readMethodInvocation(invokedMethodXMLSerialized);
		// test retrieving of method name
		assertEquals(DEFAULT_METHOD_NAME, toInvoke.getMethodName());
		// test retrieving of parameters
		assertArraysEquals(byteValues, toInvoke.getParameters());
	}
	
	
	/**
	 * Test reflection on Byte values.
	 * The suitable method is invoked and returns parameters used for this method invocation as an Object[].
	 * Then, Object[] retrieved is compared with initial Object[].
	 * 
	 */
	public void testBytesInvocation(){
		// invoke replyBytes method
		Object[] result = methodInvocation(RMAITestManager.testBytesMethodName, byteValues);
		// test retrieving of parameters
		assertArraysEquals(byteValues, result);	
	}
	
	/**
	 * Test whole RMAI process (serialization, deserialization and invocation) on Bytes values.
	 * First, byte values contained, in an Object[] array, and method name are serialized.
	 * Then, they are deserialize to test serialization/deserialization process.
	 * After, suitable method is invoked and returns parameters used for this method invocation as an Object[].
	 * Eventually, Object[] retrieved is compared with initial Object[].
	 * 
	 */
	public void testBytes()throws Exception
	{
		params = byteValues;
		String invokedMethodXMLSerialized = rmaiUtils.methodInvocation(RMAITestManager.testBytesMethodName, params);
		Object[] result = treatInvokedMethodXMLSerialized(invokedMethodXMLSerialized);
		assertArraysEquals(result,params);
	}
	
	
	/********************************** Tests on Shorts ********************************/
	
	/**
	 * Test serialization/deserialization process for Short values.
	 * Byte values contained in an Object[] array, and a method name are serialized to a String.
	 * Then, an InvokedMethod is create from parsing of the serialized string.
	 * Eventually, initial method name and parameters are compared with retrieved ones.
	 * 
	 */
	public void testShortsSerialization() throws Exception, ClassNotFoundException,IllegalAccessException,InstantiationException{
		// serialize method name and param
		String invokedMethodXMLSerialized = rmaiUtils.methodInvocation(DEFAULT_METHOD_NAME, shortValues);
		// deserialize the XML into an InvokeMethod object
		InvokedMethod toInvoke = rmaiUtils.readMethodInvocation(invokedMethodXMLSerialized);
		// test retrieving of method name
		assertEquals(DEFAULT_METHOD_NAME, toInvoke.getMethodName());
		// test retrieving of parameters
		assertArraysEquals(shortValues, toInvoke.getParameters());
	}
	
	
	/**
	 * Test reflection on Short values.
	 * The suitable method is invoked and returns parameters used for this method invocation as an Object[].
	 * Then, Object[] retrieved is compared with initial Object[].
	 * 
	 */
	public void testShortsInvocation(){
		// invoke replyShorts method
		Object[] result = methodInvocation(RMAITestManager.testShortsMethodName, shortValues);
		// test retrieving of parameters
		assertArraysEquals(shortValues, result);	
	}
	
	
	/**
	 * Test whole RMAI process (serialization, deserialization and invocation) on Shorts values.
	 * First, short values contained, in an Object[] array, and method name are serialized.
	 * Then, they are deserialize to test serialization/deserialization process.
	 * After, suitable method is invoked and returns parameters used for this method invocation as an Object[].
	 * Eventually, Object[] retrieved is compared with initial Object[].
	 * 
	 */
	public void testShorts()throws Exception
	{
		params = shortValues;
		String invokedMethodXMLSerialized = rmaiUtils.methodInvocation(RMAITestManager.testShortsMethodName, params);
		Object[] result = treatInvokedMethodXMLSerialized(invokedMethodXMLSerialized);
		assertArraysEquals(result,params);
	}
	
	
	/********************************** Tests on Integers ********************************/
	
	/**
	 * Test serialization/deserialization process for Integers values.
	 * Byte values contained in an Object[] array, and a method name are serialized to a String.
	 * Then, an InvokedMethod is create from parsing of the serialized string.
	 * Eventually, initial method name and parameters are compared with retrieved ones.
	 * 
	 */
	public void testIntegersSerialization() throws Exception, ClassNotFoundException,IllegalAccessException,InstantiationException{
		// serialize method name and param
		String invokedMethodXMLSerialized = rmaiUtils.methodInvocation(DEFAULT_METHOD_NAME, integerValues);
		// deserialize the XML into an InvokeMethod object
		InvokedMethod toInvoke = rmaiUtils.readMethodInvocation(invokedMethodXMLSerialized);
		// test retrieving of method name
		assertEquals(DEFAULT_METHOD_NAME, toInvoke.getMethodName());
		// test retrieving of parameters
		assertArraysEquals(integerValues, toInvoke.getParameters());
	}
	
	
	/**
	 * Test reflection on Integer values.
	 * The suitable method is invoked and returns parameters used for this method invocation as an Object[].
	 * Then, Object[] retrieved is compared with initial Object[].
	 * 
	 */
	public void testIntegersInvocation(){
		// invoke replyIntegers method
		Object[] result = methodInvocation(RMAITestManager.testIntegersMethodName, integerValues);
		// test retrieving of parameters
		assertArraysEquals(integerValues, result);	
	}
	

	/**
	 * Test whole RMAI process (serialization, deserialization and invocation) on Integers values.
	 * First, integer values contained, in an Object[] array, and method name are serialized.
	 * Then, they are deserialize to test serialization/deserialization process.
	 * After, suitable method is invoked and returns parameters used for this method invocation as an Object[].
	 * Eventually, Object[] retrieved is compared with initial Object[].
	 * 
	 */
	public void testIntegers()throws Exception
	{
		params = integerValues;
		String invokedMethodXMLSerialized = rmaiUtils.methodInvocation(RMAITestManager.testIntegersMethodName, params);
		Object[] result = treatInvokedMethodXMLSerialized(invokedMethodXMLSerialized);
		assertArraysEquals(result,params);
	}
	
	
	/********************************** Tests on Longs ********************************/
	
	/**
	 * Test serialization/deserialization process for Longs values.
	 * Byte values contained in an Object[] array, and a method name are serialized to a String.
	 * Then, an InvokedMethod is create from parsing of the serialized string.
	 * Eventually, initial method name and parameters are compared with retrieved ones.
	 * 
	 */
	public void testLongsSerialization() throws Exception, ClassNotFoundException,IllegalAccessException,InstantiationException{
		// serialize method name and param
		String invokedMethodXMLSerialized = rmaiUtils.methodInvocation(DEFAULT_METHOD_NAME, longValues);
		// deserialize the XML into an InvokeMethod object
		InvokedMethod toInvoke = rmaiUtils.readMethodInvocation(invokedMethodXMLSerialized);
		// test retrieving of method name
		assertEquals(DEFAULT_METHOD_NAME, toInvoke.getMethodName());
		// test retrieving of parameters
		assertArraysEquals(longValues, toInvoke.getParameters());
	}
	
	
	/**
	 * Test reflection on Long values.
	 * The suitable method is invoked and returns parameters used for this method invocation as an Object[].
	 * Then, Object[] retrieved is compared with initial Object[].
	 * 
	 */
	public void testLongsInvocation(){
		// invoke replyLongs method
		Object[] result = methodInvocation(RMAITestManager.testLongsMethodName, longValues);
		// test retrieving of parameters
		assertArraysEquals(longValues, result);	
	}
	

	/**
	 * Test whole RMAI process (serialization, deserialization and invocation) on Longs values.
	 * First, long values contained, in an Object[] array, and method name are serialized.
	 * Then, they are deserialize to test serialization/deserialization process.
	 * After, suitable method is invoked and returns parameters used for this method invocation as an Object[].
	 * Eventually, Object[] retrieved is compared with initial Object[].
	 * 
	 */
	public void testLongs()throws Exception
	{
		params = longValues;
		String invokedMethodXMLSerialized = rmaiUtils.methodInvocation(RMAITestManager.testLongsMethodName, params);
		Object[] result = treatInvokedMethodXMLSerialized(invokedMethodXMLSerialized);
		assertArraysEquals(result,params);
	}
	
	
	/********************************** Tests on Floats ********************************/
	
	/**
	 * Test serialization/deserialization process for Floats values.
	 * Byte values contained in an Object[] array, and a method name are serialized to a String.
	 * Then, an InvokedMethod is create from parsing of the serialized string.
	 * Eventually, initial method name and parameters are compared with retrieved ones.
	 * 
	 */
	public void testFloatsSerialization() throws Exception, ClassNotFoundException,IllegalAccessException,InstantiationException{
		// serialize method name and param
		String invokedMethodXMLSerialized = rmaiUtils.methodInvocation(DEFAULT_METHOD_NAME, floatValues);
		// deserialize the XML into an InvokeMethod object
		InvokedMethod toInvoke = rmaiUtils.readMethodInvocation(invokedMethodXMLSerialized);
		// test retrieving of method name
		assertEquals(DEFAULT_METHOD_NAME, toInvoke.getMethodName());
		// test retrieving of parameters
		assertArraysEquals(floatValues, toInvoke.getParameters());
	}
	
	
	/**
	 * Test reflection on Float values.
	 * The suitable method is invoked and returns parameters used for this method invocation as an Object[].
	 * Then, Object[] retrieved is compared with initial Object[].
	 * 
	 */
	public void testFloatsInvocation(){
		// invoke replyFloats method
		Object[] result = methodInvocation(RMAITestManager.testFloatsMethodName, floatValues);
		// test retrieving of parameters
		assertArraysEquals(floatValues, result);	
	}
	

	/**
	 * Test whole RMAI process (serialization, deserialization and invocation) on Float values.
	 * First, float values contained, in an Object[] array, and method name are serialized.
	 * Then, they are deserialize to test serialization/deserialization process.
	 * After, suitable method is invoked and returns parameters used for this method invocation as an Object[].
	 * Eventually, Object[] retrieved is compared with initial Object[].
	 * 
	 */
	public void testFloats()throws Exception
	{
		params = floatValues;
		String invokedMethodXMLSerialized = rmaiUtils.methodInvocation(RMAITestManager.testFloatsMethodName, params);
		Object[] result = treatInvokedMethodXMLSerialized(invokedMethodXMLSerialized);
		assertArraysEquals(result,params);
	}
	
	
	/********************************** Tests on Doubles ********************************/
	
	/**
	 * Test serialization/deserialization process for Doubles values.
	 * Byte values contained in an Object[] array, and a method name are serialized to a String.
	 * Then, an InvokedMethod is create from parsing of the serialized string.
	 * Eventually, initial method name and parameters are compared with retrieved ones.
	 * 
	 */
	public void testDoublesSerialization() throws Exception, ClassNotFoundException,IllegalAccessException,InstantiationException{
		// serialize method name and param
		String invokedMethodXMLSerialized = rmaiUtils.methodInvocation(DEFAULT_METHOD_NAME, doubleValues);
		// deserialize the XML into an InvokeMethod object
		InvokedMethod toInvoke = rmaiUtils.readMethodInvocation(invokedMethodXMLSerialized);
		// test retrieving of method name
		assertEquals(DEFAULT_METHOD_NAME, toInvoke.getMethodName());
		// test retrieving of parameters
		assertArraysEquals(doubleValues, toInvoke.getParameters());
	}
	
	
	/**
	 * Test reflection on Double values.
	 * The suitable method is invoked and returns parameters used for this method invocation as an Object[].
	 * Then, Object[] retrieved is compared with initial Object[].
	 * 
	 */
	public void testDoublesInvocation(){
		// invoke replyDoubles method
		Object[] result = methodInvocation(RMAITestManager.testDoublesMethodName, doubleValues);
		// test retrieving of parameters
		assertArraysEquals(doubleValues, result);		
	}
	

	/**
	 * Test whole RMAI process (serialization, deserialization and invocation) on Double values.
	 * First, double values contained, in an Object[] array, and method name are serialized.
	 * Then, they are deserialize to test serialization/deserialization process.
	 * After, suitable method is invoked and returns parameters used for this method invocation as an Object[].
	 * Eventually, Object[] retrieved is compared with initial Object[].
	 * 
	 */
	public void testDoubles()throws Exception
	{
		params = doubleValues;
		String invokedMethodXMLSerialized = rmaiUtils.methodInvocation(RMAITestManager.testDoublesMethodName, params);
		Object[] result = treatInvokedMethodXMLSerialized(invokedMethodXMLSerialized);
		assertArraysEquals(result,params);
	}
	
	
	/********************************** Tests on Booleans ********************************/
	
	/**
	 * Test serialization/deserialization process for Booleans values.
	 * Byte values contained in an Object[] array, and a method name are serialized to a String.
	 * Then, an InvokedMethod is create from parsing of the serialized string.
	 * Eventually, initial method name and parameters are compared with retrieved ones.
	 * 
	 */
	public void testBooleansSerialization() throws Exception, ClassNotFoundException,IllegalAccessException,InstantiationException{
		// serialize method name and param
		String invokedMethodXMLSerialized = rmaiUtils.methodInvocation(DEFAULT_METHOD_NAME, booleanValues);
		// deserialize the XML into an InvokeMethod object
		InvokedMethod toInvoke = rmaiUtils.readMethodInvocation(invokedMethodXMLSerialized);
		// test retrieving of method name
		assertEquals(DEFAULT_METHOD_NAME, toInvoke.getMethodName());
		// test retrieving of parameters
		assertArraysEquals(booleanValues, toInvoke.getParameters());
	}
	
	
	/**
	 * Test reflection on Booleans values.
	 * The suitable method is invoked and returns parameters used for this method invocation as an Object[].
	 * Then, Object[] retrieved is compared with initial Object[].
	 * 
	 */
	public void testBooleansInvocation(){
		// invoke replyBooleans method
		Object[] result = methodInvocation(RMAITestManager.testBooleansMethodName, booleanValues);
		// test retrieving of parameters
		assertArraysEquals(booleanValues, result);		
	}
	

	/**
	 * Test whole RMAI process (serialization, deserialization and invocation) on Boolean values.
	 * First, boolean values contained, in an Object[] array, and method name are serialized.
	 * Then, they are deserialize to test serialization/deserialization process.
	 * After, suitable method is invoked and returns parameters used for this method invocation as an Object[].
	 * Eventually, Object[] retrieved is compared with initial Object[].
	 * 
	 */
	public void testBooleans()throws Exception
	{
		params = booleanValues;
		String invokedMethodXMLSerialized = rmaiUtils.methodInvocation(RMAITestManager.testBooleansMethodName, params);
		Object[] result = treatInvokedMethodXMLSerialized(invokedMethodXMLSerialized);
		assertArraysEquals(result,params);
	}
	
	
	/********************************** Tests on Strings ********************************/
	
	/**
	 * Test serialization/deserialization process for Strings values.
	 * Byte values contained in an Object[] array, and a method name are serialized to a String.
	 * Then, an InvokedMethod is create from parsing of the serialized string.
	 * Eventually, initial method name and parameters are compared with retrieved ones.
	 * 
	 */
	public void testStringsSerialization() throws Exception, ClassNotFoundException,IllegalAccessException,InstantiationException{
		// serialize method name and param
		String invokedMethodXMLSerialized = rmaiUtils.methodInvocation(DEFAULT_METHOD_NAME, stringValues);
		// deserialize the XML into an InvokeMethod object
		InvokedMethod toInvoke = rmaiUtils.readMethodInvocation(invokedMethodXMLSerialized);
		// test retrieving of method name
		assertEquals(DEFAULT_METHOD_NAME, toInvoke.getMethodName());
		// test retrieving of parameters
		assertArraysEquals(stringValues, toInvoke.getParameters());
	}
	
	
	/**
	 * Test reflection on Strings values.
	 * The suitable method is invoked and returns parameters used for this method invocation as an Object[].
	 * Then, Object[] retrieved is compared with initial Object[].
	 * 
	 */
	public void testStringsInvocation(){
		// invoke replyStrings method
		Object[] result = methodInvocation(RMAITestManager.testStringsMethodName, stringValues);
		// test retrieving of parameters
		assertArraysEquals(stringValues, result);
	}
	

	/**
	 * Test whole RMAI process (serialization, deserialization and invocation) on String values.
	 * First, string values contained, in an Object[] array, and method name are serialized.
	 * Then, they are deserialize to test serialization/deserialization process.
	 * After, suitable method is invoked and returns parameters used for this method invocation as an Object[].
	 * Eventually, Object[] retrieved is compared with initial Object[].
	 * 
	 */
	public void testStrings()throws Exception
	{
		params = stringValues;
		String invokedMethodXMLSerialized = rmaiUtils.methodInvocation(RMAITestManager.testStringsMethodName, params);
		Object[] result = treatInvokedMethodXMLSerialized(invokedMethodXMLSerialized);
		assertArraysEquals(result,params);
	}
	

	/********************************** Tests on RMAISerializable ********************************/
	
	/**
	 * Test serialization/deserialization process for RMAISerializable values.
	 * Byte values contained in an Object[] array, and a method name are serialized to a String.
	 * Then, an InvokedMethod is create from parsing of the serialized string.
	 * Eventually, initial method name and parameters are compared with retrieved ones.
	 * 
	 */
	public void testSerializableSerialization() throws Exception, ClassNotFoundException,IllegalAccessException,InstantiationException{
		// serialize method name and param
		String invokedMethodXMLSerialized = rmaiUtils.methodInvocation(DEFAULT_METHOD_NAME, serializableValues);
		// deserialize the XML into an InvokeMethod object
		InvokedMethod toInvoke = rmaiUtils.readMethodInvocation(invokedMethodXMLSerialized);
		// test retrieving of method name
		assertEquals(DEFAULT_METHOD_NAME, toInvoke.getMethodName());
		// test retrieving of parameters
		assertArraysEquals(serializableValues, toInvoke.getParameters());
	}
	
	
	/**
	 * Test reflection on RMAISerializable values.
	 * The suitable method is invoked and returns parameters used for this method invocation as an Object[].
	 * Then, Object[] retrieved is compared with initial Object[].
	 * 
	 */
	public void testSerializableInvocation(){
		// invoke replyStrings method
		Object[] result = methodInvocation(RMAITestManager.testSerializablesMethodName, serializableValues);
		// test retrieving of parameters
		assertArraysEquals(serializableValues, result);
	}
	

	/**
	 * Test whole RMAI process (serialization, deserialization and invocation) on RMAISerializable values.
	 * First, string values contained, in an Object[] array, and method name are serialized.
	 * Then, they are deserialize to test serialization/deserialization process.
	 * After, suitable method is invoked and returns parameters used for this method invocation as an Object[].
	 * Eventually, Object[] retrieved is compared with initial Object[].
	 * 
	 */
	public void testSerializable()throws Exception
	{
		params = serializableValues;
		String invokedMethodXMLSerialized = rmaiUtils.methodInvocation(RMAITestManager.testSerializablesMethodName, params);
		Object[] result = treatInvokedMethodXMLSerialized(invokedMethodXMLSerialized);
		assertArraysEquals(result,params);
	}
	
	
	/********************************** Tests on Object[] ********************************/
	
	/**
	 * Test serialization/deserialization process for Object[] values.
	 * Byte values contained in an Object[] array, and a method name are serialized to a String.
	 * Then, an InvokedMethod is create from parsing of the serialized string.
	 * Eventually, initial method name and parameters are compared with retrieved ones.
	 * 
	 */
	public void testArrayOfObjectSerialization() throws Exception, ClassNotFoundException,IllegalAccessException,InstantiationException{
		// serialize method name and param
		String invokedMethodXMLSerialized = rmaiUtils.methodInvocation(DEFAULT_METHOD_NAME, arrayOfObjectsValues);
		// deserialize the XML into an InvokeMethod object
		InvokedMethod toInvoke = rmaiUtils.readMethodInvocation(invokedMethodXMLSerialized);
		// test retrieving of method name
		assertEquals(DEFAULT_METHOD_NAME, toInvoke.getMethodName());
		// test retrieving of bytesValues
		Object[] byteArrayRetrieved = (Object[])toInvoke.getParameters()[0];
		assertArraysEquals(byteValues, byteArrayRetrieved);
		// test retrieving of shortValues
		Object[] shortArrayRetrieved = (Object[])toInvoke.getParameters()[1];
		assertArraysEquals(shortValues, shortArrayRetrieved);
		// test retrieving of integerValues
		Object[] integerArrayRetrieved = (Object[])toInvoke.getParameters()[2];
		assertArraysEquals(integerValues, integerArrayRetrieved);
		// test retrieving of longValues
		Object[] longArrayRetrieved = (Object[])toInvoke.getParameters()[3];
		assertArraysEquals(longValues, longArrayRetrieved);
		// test retrieving of floatValues
		Object[] floatArrayRetrieved = (Object[])toInvoke.getParameters()[4];
		assertArraysEquals(floatValues, floatArrayRetrieved);
		// test retrieving of doubleValues
		Object[] doubleArrayRetrieved = (Object[])toInvoke.getParameters()[5];
		assertArraysEquals(doubleValues, doubleArrayRetrieved);
		// test retrieving of booleanValues
		Object[] booleanArrayRetrieved = (Object[])toInvoke.getParameters()[6];
		assertArraysEquals(booleanValues, booleanArrayRetrieved);
		// test retrieving of stringValues
		Object[] stringArrayRetrieved = (Object[])toInvoke.getParameters()[7];
		assertArraysEquals(stringValues, stringArrayRetrieved);
	}
	
	
	/**
	 * Test reflection on Object[] values.
	 * The suitable method is invoked and returns parameters used for this method invocation as an Object[].
	 * Then, elements of the Object[] retrieved are compared with elements of the initial Object[].
	 * 
	 */
	public void testArrayOfObjectInvocation(){
		// invoke replyArrayOfObjects method
		Object[] result = methodInvocation(RMAITestManager.testArrayOfObjectsMethodName,arrayOfObjectsValues);
		// test retrieving of bytesValues
		Object[] byteArrayRetrieved = (Object[])result[0];
		assertArraysEquals(byteValues, byteArrayRetrieved);
		// test retrieving of shortValues
		Object[] shortArrayRetrieved = (Object[])result[1];
		assertArraysEquals(shortValues, shortArrayRetrieved);
		// test retrieving of integerValues
		Object[] integerArrayRetrieved = (Object[])result[2];
		assertArraysEquals(integerValues, integerArrayRetrieved);
		// test retrieving of longValues
		Object[] longArrayRetrieved = (Object[])result[3];
		assertArraysEquals(longValues, longArrayRetrieved);
		// test retrieving of floatValues
		Object[] floatArrayRetrieved = (Object[])result[4];
		assertArraysEquals(floatValues, floatArrayRetrieved);
		// test retrieving of doubleValues
		Object[] doubleArrayRetrieved = (Object[])result[5];
		assertArraysEquals(doubleValues, doubleArrayRetrieved);
		// test retrieving of booleanValues
		Object[] booleanArrayRetrieved = (Object[])result[6];
		assertArraysEquals(booleanValues, booleanArrayRetrieved);
		// test retrieving of stringValues
		Object[] stringArrayRetrieved = (Object[])result[7];
		assertArraysEquals(stringValues, stringArrayRetrieved);
	}
	

	/**
	 * Test whole RMAI process (serialization, deserialization and invocation) on Object[] values.
	 * First, Object[] values contained in an Object[] array, and method name are serialized.
	 * Then, they are deserialize to test serialization/deserialization process.
	 * After, suitable method is invoked and returns parameters used for this method invocation as an Object[].
	 * Eventually, elements of the Object[] retrieved are compared with elements of the initial Object[].
	 * 
	 */
	public void testArrayOfObjects()throws Exception
	{
		params = arrayOfObjectsValues;
		String invokedMethodXMLSerialized = rmaiUtils.methodInvocation(RMAITestManager.testArrayOfObjectsMethodName, params);
		Object[] result = treatInvokedMethodXMLSerialized(invokedMethodXMLSerialized);
		// test retrieving of bytesValues
		Object[] byteArrayRetrieved = (Object[])result[0];
		assertArraysEquals(byteValues, byteArrayRetrieved);
		// test retrieving of shortValues
		Object[] shortArrayRetrieved = (Object[])result[1];
		assertArraysEquals(shortValues, shortArrayRetrieved);
		// test retrieving of integerValues
		Object[] integerArrayRetrieved = (Object[])result[2];
		assertArraysEquals(integerValues, integerArrayRetrieved);
		// test retrieving of longValues
		Object[] longArrayRetrieved = (Object[])result[3];
		assertArraysEquals(longValues, longArrayRetrieved);
		// test retrieving of floatValues
		Object[] floatArrayRetrieved = (Object[])result[4];
		assertArraysEquals(floatValues, floatArrayRetrieved);
		// test retrieving of doubleValues
		Object[] doubleArrayRetrieved = (Object[])result[5];
		assertArraysEquals(doubleValues, doubleArrayRetrieved);
		// test retrieving of booleanValues
		Object[] booleanArrayRetrieved = (Object[])result[6];
		assertArraysEquals(booleanValues, booleanArrayRetrieved);
		// test retrieving of stringValues
		Object[] stringArrayRetrieved = (Object[])result[7];
		assertArraysEquals(stringValues, stringArrayRetrieved);
	}
	

	/********************************** End of public tests methods ********************************/
	
	/*
	 * This method tests whether :
	 * - both arrays in parameters have the same length
	 * - every Object contained in an Object[] is equal (using equals() method)
	 * 	to the Object located at the same index in the other Object[].
	 */
	private void assertArraysEquals(Object[] array1, Object[] array2){
		assertEquals(array1.length, array2.length);
		for (int i = 0; i < array1.length; i++){
			assertEquals(array1[i], array2[i]);
		}
	}

	
	/*
	 * This method deserialize informations contained in a String, and invoke the suitable methods,
	 * with suitable parameters, depending on informations retrieved.
	 * The method invoked return an Object[].
	 */
	private static Object[] treatInvokedMethodXMLSerialized(String invokedMethodXMLSerialized){
		try{
			// deserialize the XML
			InvokedMethod toInvoke = rmaiUtils.readMethodInvocation(invokedMethodXMLSerialized);
			// get the class on which method should be invoked, i.e. RMAITestManager class
			Class<?> rmaiTestManagerClass = RMAITestManager.instance.getClass();
			// get a Method object matching with informations included in toInvoke ...
			Method rmaiMethod = rmaiTestManagerClass.getMethod(toInvoke.getMethodName(),toInvoke.getParametersTypes());
			// invoke the method on the rmai test manager with the suitable arguments
			Object result = rmaiMethod.invoke(RMAITestManager.instance, toInvoke.getParameters());
			return (Object[])result;
		} catch (ClassNotFoundException e){
			e.printStackTrace();
		} catch (InstantiationException e){
			e.printStackTrace();
		} catch (NoSuchMethodException e){
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		} catch (SecurityException e){
			e.printStackTrace();
		} catch (Exception e){
			e.printStackTrace();
		}
		return null;
	}

	
	/*
	 * This method invoke a method with suitable parameters, and return result of the invocation as an Object[].
	 */
	private static Object[] methodInvocation(String methodName, Object[] parameters){
		// get the class on which method should be invoked, i.e. RMAITestManager class
		Class<?> rmaiTestManagerClass = RMAITestManager.instance.getClass();
		// get the method of this class that should be invoked
		Method rmaiMethod;
		try {
			// get the suitable Method object
			rmaiMethod = rmaiTestManagerClass.getMethod(methodName,getParametersTypes(parameters));
			// invoke the method on the rmai test manager with the suitable arguments
			Object result = rmaiMethod.invoke(RMAITestManager.instance, parameters);
			return (Object[])result;
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	
	/* 
	 * Get an array of classes representing the types of an Object[].
	 * This method is needed to provide the suitable arguments to the Method class, in order to invoke
	 * the required method.
	 * 
	 * @return 	Class[]	a Class array matching with parameters
	 * 					Example: 
	 * 					Object[] parameters = {"hello",2,true} return
	 * 					Class[] types ={String,Integer,Boolean)
	 * 								
	 */
	private static Class[] getParametersTypes(Object[] parameters) {
		Class[] parametersType = new Class[parameters.length];
		for (int i = 0; i < parametersType.length; i++) {
			// get the type of the parameters
			parametersType[i] = parameters[i].getClass();
		}
		return parametersType;
	}

	
	/* 
	 * RMAI methods tested are contained in this class.
	 * Contrary to real RMAI, this methods are not of type void.
	 */
	private static class RMAITestManager{
		
		private final static RMAITestManager instance = new RMAITestManager();
		
		public final static String testBytesMethodName 			= "replyBytes";
		public final static String testShortsMethodName 		= "replyShorts";
		public final static String testIntegersMethodName 		= "replyIntegers";
		public final static String testLongsMethodName 			= "replyLongs";
		public final static String testFloatsMethodName 		= "replyFloats";
		public final static String testDoublesMethodName 		= "replyDoubles";
		public final static String testBooleansMethodName 		= "replyBooleans";
		public final static String testStringsMethodName 		= "replyStrings";
		public final static String testSerializablesMethodName	= "replySerializables";
		public final static String testArrayOfObjectsMethodName = "replyArrayOfObjects";

		private RMAITestManager(){}

		@SuppressWarnings("unused")
		public RMAITestManager getInstance(){
			return instance;
		}
		
		
		@SuppressWarnings("unused")
		public Object[] replyBytes(Byte obj1, Byte obj2, Byte obj3){
			return new Object[]{obj1,obj2,obj3};
		}
		
		
		@SuppressWarnings("unused")
		public Object[] replyShorts(Short obj1, Short obj2, Short obj3){
			return new Object[]{obj1,obj2,obj3};
		}
		
		
		@SuppressWarnings("unused")
		public Object[] replyIntegers(Integer obj1, Integer obj2, Integer obj3){
			return new Object[]{obj1,obj2,obj3};
		}
		
		
		@SuppressWarnings("unused")
		public Object[] replyLongs(Long obj1, Long obj2, Long obj3){
			return new Object[]{obj1,obj2,obj3};
		}
		
		
		@SuppressWarnings("unused")
		public Object[] replyFloats(Float obj1, Float obj2, Float obj3, Float obj4, 
									Float obj5, Float obj6){
			return new Object[]{obj1,obj2,obj3,obj4,obj5,obj6};
		}
		
		
		@SuppressWarnings("unused")
		public Object[] replyDoubles(Double obj1, Double obj2, Double obj3, Double obj4,
									Double obj5, Double obj6){
			return new Object[]{obj1,obj2,obj3,obj4,obj5,obj6};
		}
		
		
		@SuppressWarnings("unused")
		public Object[] replyBooleans(Boolean obj1, Boolean obj2){
			return new Object[]{obj1,obj2};
		}
		
		
		@SuppressWarnings("unused")
		public Object[] replyStrings(String obj1, String obj2/*,String obj3, String obj4,String obj5, String obj6*/){
			return new Object[]{obj1,obj2/*,obj3,obj4,obj5,obj6*/};
		}
		
		
		@SuppressWarnings("unused")
		public Object[] replySerializables(MyRMAIObject obj1,MyRMAIObject obj2){
			return new Object[]{obj1,obj2};
		}
		
		
		@SuppressWarnings("unused")
		public Object[] replyArrayOfObjects(Object[] obj1, Object[] obj2, Object[] obj3, Object[] obj4,
											Object[] obj5, Object[] obj6, Object[] obj7, Object[] obj8){
			return new Object[]{obj1,obj2,obj3,obj4,obj5,obj6,obj7,obj8};
		}

	}
}
